#!/usr/local/bin/python
#
# $Id: //projects/zoe/sol.py#2 $ $Date: 2002/08/09 $

"""
A demo of a static view of a (very poorly approximated) Solar System.
"""

import math

from OpenGL.GL   import * # strange, but canonical PyOpenGL import
from OpenGL.GLU  import *
from OpenGL.GLUT import *

import zoe


# Data consists of:
#
#    volumetric mean radius, semimajor axis, inclination, eccentricity

OBJECT_DATA = {'Mercury': (2439.7e3, 57.91e9, 7.00, 0.2056),
               'Venus': (6051.8e3, 108.21e9, 3.39, 0.0067),
               'Earth': (6371.0e3, 149.60e9, 0.000, 0.0167),
               'Mars': (3390e3, 227.91e9, 1.850, 0.0935),
               'Jupiter': (69911e3, 778.57e9, 1.304, 0.0489),
               'Saturn': (58232e3, 1433.53e9, 2.485, 0.0565),
               'Uranus': (25362e3, 2872.46e9, 0.772, 0.0457),
               'Neptune': (24624e3, 4495.06e9, 1.769, 0.0113)}
               #'Pluto': (1195e3, 5869.66e9, 17.16, 0.2444)}


class Body(zoe.Object):

    """Draw the body as an orbit (inclination and eccentricity are
    presently unused."""
    
    def __init__(self, name, r, a, i, e):
        self.name = name
        self.r = r
        self.a = a
        self.i = i
        self.e = e

    def display(self):
        DIVISIONS = 32
        glColor3f(1.0, 1.0, 1.0)
        glBegin(GL_LINE_STRIP)
        for i in range(DIVISIONS + 1):
            theta = zoe.twoPi*i/DIVISIONS
            dx, dy = self.a*math.cos(theta), self.a*math.sin(theta)
            glVertex3f(dx, dy, 0.0)
        glEnd()
        glColor3f(1.0, 1.0, 0.0)
        r = self.r
        glBegin(GL_LINE_STRIP)
        for i in range(DIVISIONS + 1):
            theta = zoe.twoPi*i/DIVISIONS
            dx, dy = r*math.cos(theta), r*math.sin(theta)
            glVertex3f(self.a + dx, dy, 0.0)
        glEnd()
        glBegin(GL_LINE_STRIP)
        for i in range(DIVISIONS + 1):
            theta = zoe.twoPi*i/DIVISIONS
            dx, dy = r*math.cos(theta), r*math.sin(theta)
            glVertex3f(self.a, dx, dy)
        glEnd()
        glBegin(GL_LINE_STRIP)
        for i in range(DIVISIONS + 1):
            theta = zoe.twoPi*i/DIVISIONS
            dx, dy = r*math.cos(theta), r*math.sin(theta)
            glVertex3f(self.a + dy, 0.0, dx)
        glEnd()


def main():
    engine = zoe.Engine()
    engine.setup()
    engine.camera.zoomOut(4e10)
    engine.add(zoe.AxesObject(1e12))
    engine.add(zoe.GridObject(150e9, 1e12))
    engine.add(Body('Sun', 696000e3, 0.0, 0.0, 0.0))
    for name, (r, a, i, e) in OBJECT_DATA.items():
        engine.add(Body(name, r, a, i, e))
    engine.go()

if __name__ == '__main__': main()
